@@ -150,14 +150,14 @@ public final class Java
         @Override
         public LispObject execute(LispObject arg)
         {
-	    return JavaObject.getInstance(javaClass(arg, JavaClassLoader.getCurrentClassLoader()));
+            return JavaObject.getInstance(javaClass(arg, JavaClassLoader.getCurrentClassLoader()));
         }
 
         @Override
         public LispObject execute(LispObject className, LispObject classLoader)
         {
-	    ClassLoader loader = (ClassLoader) classLoader.javaInstance(ClassLoader.class);
-	    return JavaObject.getInstance(javaClass(className, loader));
+            ClassLoader loader = (ClassLoader) classLoader.javaInstance(ClassLoader.class);
+            return JavaObject.getInstance(javaClass(className, loader));
         }
     };
 
@@ -237,7 +237,7 @@ public final class Java
     private static final Primitive JFIELD = new pf_jfield();
     @DocString(name="jfield",
     args="class-ref-or-field field-or-instance &optional instance value",
-    doc="Retrieves or modifies a field in a Java class or instance.\n\n"+
+    doc="Retrieves or modifies a public field in a Java class or instance.\n\n"+
         "Supported argument patterns:\n\n"+
         "   Case 1: class-ref  field-name:\n"+
         "      Retrieves the value of a static field.\n\n"+
@@ -274,8 +274,8 @@ public final class Java
     private static final Primitive JFIELD_RAW = new pf_jfield_raw();
     @DocString(name="jfield",
     args="class-ref-or-field field-or-instance &optional instance value",
-    doc="Retrieves or modifies a field in a Java class or instance. Does not\n"+
-        "attempt to coerce its value or the result into a Lisp object.\n\n"+
+    doc="Retrieves or modifies a public field in a Java class or instance without\n"+
+        "attempting to coerce its value or the result into a Lisp object.\n\n"+
         "Supported argument patterns:\n\n"+
         "   Case 1: class-ref  field-name:\n"+
         "      Retrieves the value of a static field.\n\n"+
@@ -433,6 +433,7 @@ public final class Java
         }
         try {
             Method m = null;
+            Class c = null;
             LispObject methodRef = args[0];
             List<Method> staticMethods = new ArrayList<Method>();
             String methodName = null;
@@ -446,7 +447,7 @@ public final class Java
                     error(new LispError(methodRef +  "is not a valid reference to a Method"));
                 }
             } else if (methodRef instanceof AbstractString) {
-                Class c = javaClass(args[1]);
+                c = javaClass(args[1]);
                 if (c != null) {
                     methodName = methodRef.getStringValue();
                     Method[] methods = c.getMethods();
@@ -465,8 +466,25 @@ public final class Java
                 m = findMethod(staticMethods.toArray(new Method[staticMethods.size()]),
                                methodName, args, 2);
             }
-            if (m == null)
-                error(new LispError("no such method"));
+            if (m == null) {
+              StringBuilder sb = new StringBuilder("No such static method: ");
+              String className = "<unknown>";
+              if (c != null) {
+                className = c.getName();
+              }
+              sb.append(className);
+              sb.append('.');
+              sb.append(methodName);
+              sb.append('(');
+              for (int i = 2; i < args.length; i++) {
+                LispObject argClass = Symbol.JCLASS_OF.execute(args[i]);
+                sb.append(argClass.princToString());
+                if (i < args.length - 1)
+                  sb.append(',');
+              }
+              sb.append(')');
+              error(new LispError(sb.toString()));
+            }
 
             Object[] methodArgs = new Object[args.length-2];
             Class[] argTypes = m.getParameterTypes();
@@ -484,10 +502,16 @@ public final class Java
             Object result = null;
             if (!m.isVarArgs()) {
               result = m.invoke(null, methodArgs);
-            } else {
+            } else if (argTypes.length == 1) {
               result = m.invoke(null, (Object)methodArgs);
+            } else {
+              Object[] objectArgs = new Object[methodArgs.length];
+              for (int i = 0; i < methodArgs.length; i++) {
+                objectArgs[i] = methodArgs[i];
+              }
+              result = m.invoke(null, objectArgs);
             }
-	    return JavaObject.getInstance(result, translate, m.getReturnType());
+            return JavaObject.getInstance(result, translate, m.getReturnType());
         }
         catch (ControlTransfer c) {
             throw c;
@@ -563,18 +587,18 @@ public final class Java
             LispObject classRef = args[0];
             try {
                 Constructor constructor;
-		if(classRef instanceof AbstractString) {
-		    constructor = findConstructor(javaClass(classRef), args);
-		} else {
-		    Object object = JavaObject.getObject(classRef);
-		    if(object instanceof Constructor) {
-			constructor = (Constructor) object;
-		    } else if(object instanceof Class<?>) {
-			constructor = findConstructor((Class<?>) object, args);
-		    } else {
-			return error(new LispError(classRef.princToString() + " is neither a Constructor nor a Class"));
-		    }
-		}
+                if(classRef instanceof AbstractString) {
+                    constructor = findConstructor(javaClass(classRef), args);
+                } else {
+                    Object object = JavaObject.getObject(classRef);
+                    if(object instanceof Constructor) {
+                        constructor = (Constructor) object;
+                    } else if(object instanceof Class<?>) {
+                        constructor = findConstructor((Class<?>) object, args);
+                    } else {
+                        return error(new LispError(classRef.princToString() + " is neither a Constructor nor a Class"));
+                    }
+                }
                 Class[] argTypes = constructor.getParameterTypes();
                 Object[] initargs = new Object[args.length-1];
                 for (int i = 1; i < args.length; i++) {
@@ -728,9 +752,28 @@ public final class Java
                     a = Array.get(a, ((Integer)args[i].javaInstance()).intValue());
                 Object value = v.javaInstance();
                 int index = ((Integer)args[args.length - 1].javaInstance()).intValue();
-                if (value instanceof java.lang.Number
-                    && a.getClass().getComponentType().equals(Byte.TYPE)) {
-                    Array.setByte(a, index, ((java.lang.Number)value).byteValue());
+                if (value instanceof java.lang.Number) {
+                  Class type = a.getClass().getComponentType();
+                  Number number = (java.lang.Number)value;
+                  if (type.equals(Byte.TYPE)) {
+                    Array.setByte(a, index, number.byteValue());
+                  } else if (type.equals(Short.TYPE)) {
+                    Array.setShort(a, index, number.shortValue()); 
+                  } else if (type.equals(Character.TYPE)) {
+                    Array.setChar(a, index, (char)number.intValue()); // FIXME: value out of range
+                  } else if (type.equals(Integer.TYPE)) {
+                    Array.setInt(a, index, number.intValue()); 
+                  } else if (type.equals(Long.TYPE)) {
+                    Array.setLong(a, index, number.longValue()); 
+                  } else if (type.equals(Float.TYPE)) {
+                    Array.setFloat(a, index, number.floatValue()); 
+                  } else if (type.equals(Double.TYPE)) {
+                    Array.setDouble(a, index, number.doubleValue()); 
+                  } else if (type.equals(Boolean.TYPE)) {
+                    Array.setBoolean(a, index, number.equals(0) ? false : true); 
+                  } else {
+                    Array.set(a, index, value);
+                  }
                 } else {
                     Array.set(a, index, value);
                 }
@@ -892,9 +935,9 @@ public final class Java
             } else
                 method = (Method) JavaObject.getObject(methodArg);
             Class<?>[] argTypes = (Class<?>[])method.getParameterTypes();
-	    if(argTypes.length != args.length - 2) {
-		return error(new WrongNumberOfArgumentsException("Wrong number of arguments for " + method + ": expected " + argTypes.length + ", got " + (args.length - 2)));
-	    }
+            if(argTypes.length != args.length - 2) {
+                return error(new WrongNumberOfArgumentsException("Wrong number of arguments for " + method + ": expected " + argTypes.length + ", got " + (args.length - 2)));
+            }
             methodArgs = new Object[argTypes.length];
             for (int i = 2; i < args.length; i++) {
               LispObject arg = args[i];
@@ -909,9 +952,9 @@ public final class Java
             if (!method.isAccessible()) {
                  // Possible for static member classes: see #229
                  if (Modifier.isPublic(method.getModifiers())) { 
-    	              method.setAccessible(true);
+                      method.setAccessible(true);
                  }
-	    }
+            }
             return JavaObject.getInstance(method.invoke(instance, methodArgs),
                                           translate,
                                           method.getReturnType());
@@ -938,11 +981,11 @@ public final class Java
     }
 
     private static Object[] translateMethodArguments(LispObject[] args) {
-	return translateMethodArguments(args, 0);
+        return translateMethodArguments(args, 0);
     }
 
     private static Object[] translateMethodArguments(LispObject[] args, int offs) {
-	int argCount = args.length - offs;
+        int argCount = args.length - offs;
         Object[] javaArgs = new Object[argCount];
         for (int i = 0; i < argCount; ++i) {
           Object x = args[i + offs];
@@ -954,7 +997,7 @@ public final class Java
             javaArgs[i] = ((LispObject) x).javaInstance();
           }
         }
-	return javaArgs;
+        return javaArgs;
     }
 
     private static Method findMethod(Method[] methods, String methodName, Object[] javaArgs) {
@@ -989,11 +1032,11 @@ public final class Java
             actualClass = instance.getClass();
             if(intendedClass != actualClass) { 
                 method = findMethod(actualClass, methodName, methodArgs);
-		if (method != null) {
-		   if (isMethodCallableOnInstance(actualClass, method)) {
-		      return method;
-		   }
-		}
+                if (method != null) {
+                   if (isMethodCallableOnInstance(actualClass, method)) {
+                      return method;
+                   }
+                }
             }
         }
         return method;
@@ -1001,10 +1044,10 @@ public final class Java
     
     private static boolean isMethodCallableOnInstance(Class instance, Method method) {
        if (Modifier.isPublic(method.getModifiers())) {
-	  return true;
+          return true;
        }
        if (instance.isMemberClass()) {
-	  return isMethodCallableOnInstance(instance.getEnclosingClass(), method);
+          return isMethodCallableOnInstance(instance.getEnclosingClass(), method);
        }
        return false;
     }
@@ -1043,22 +1086,22 @@ public final class Java
             }
         }
         if (result == null) {
-	    StringBuilder sb = new StringBuilder(c.getSimpleName());
-	    sb.append('(');
-	    boolean first = true;
-	    for(Object o : javaArgs) {
-		if(first) {
-		    first = false;
-		} else {
-		    sb.append(", ");
-		}
-		if(o != null) {
-		    sb.append(o.getClass().getName());
-		} else {
-		    sb.append("<null>");
-		}
-	    }
-	    sb.append(')');
+            StringBuilder sb = new StringBuilder(c.getSimpleName());
+            sb.append('(');
+            boolean first = true;
+            for(Object o : javaArgs) {
+                if(first) {
+                    first = false;
+                } else {
+                    sb.append(", ");
+                }
+                if(o != null) {
+                    sb.append(o.getClass().getName());
+                } else {
+                    sb.append("<null>");
+                }
+            }
+            sb.append(')');
             throw new NoSuchMethodException(sb.toString());
         }
         return result;
@@ -1159,11 +1202,11 @@ public final class Java
     }
 
     public static Class<?> maybeBoxClass(Class<?> clazz) {
-	if(clazz.isPrimitive()) {
-	    return getBoxedClass(clazz);
-	} else {
-	    return clazz;
-	}
+        if(clazz.isPrimitive()) {
+            return getBoxedClass(clazz);
+        } else {
+            return clazz;
+        }
     }
     
     private static Class<?> getBoxedClass(Class<?> clazz) {
@@ -1308,13 +1351,13 @@ public final class Java
         @Override
         public LispObject execute(LispObject javaObject, LispObject intendedClass)
         {
-	    Object o = javaObject.javaInstance();
-	    Class<?> c = javaClass(intendedClass);
-	    try {
-		return JavaObject.getInstance(o, c);
-	    } catch(ClassCastException e) {
+            Object o = javaObject.javaInstance();
+            Class<?> c = javaClass(intendedClass);
+            try {
+                return JavaObject.getInstance(o, c);
+            } catch(ClassCastException e) {
           return type_error(javaObject, new SimpleString(c.getName()));
-	    }
+            }
         }
     };
 
@@ -1341,28 +1384,53 @@ public final class Java
             }
             catch (StackOverflowError oos) {
                 oos.printStackTrace();
-                return error(new StorageCondition("Stack overflow."));
+                return error(new StorageCondition("Stack overflow encountered in closure execution"));
             }
         }
     };
 
     private static Class classForName(String className) {
-	return classForName(className, JavaClassLoader.getPersistentInstance());
+        return classForName(className, JavaClassLoader.getPersistentInstance());
     }
 
-    private static Class classForName(String className, ClassLoader classLoader) {
-        try {
-            return Class.forName(className, true, classLoader);
-        }
-        catch (ClassNotFoundException e) {
-	    error(new LispError("Class not found: " + className));
-	    // Not reached.
-	    return null;
+  private static Class classForName(String className,
+                                    ClassLoader classLoader) {
+    try {
+      if (!className.endsWith("[]")) {
+        return Class.forName(className, true, classLoader);
+      } else {
+        // <https://docs.oracle.com/javase/specs/jvms/se7/html/jvms-4.html#jvms-4.2>
+        if (className.startsWith("byte")) {
+          return Class.forName("[B");
+        } else if (className.startsWith("char")) {
+          return Class.forName("[C");
+        } else if (className.startsWith("double")) {
+          return Class.forName("[D");
+        } else if (className.startsWith("float")) {
+          return Class.forName("[F");
+        } else if (className.startsWith("int")) {
+          return Class.forName("[I");
+        } else if (className.startsWith("long")) {
+          return Class.forName("[J");
+        } else if (className.startsWith("short")) {
+          return Class.forName("[S");
+        } else if (className.startsWith("boolean")) {
+          return Class.forName("[Z");
+        } else {
+          final String arrayTypeName
+            = "[L" + className.substring(0, className.length() - 2) + ";";
+          return Class.forName(arrayTypeName); 
         }
+      }
+    } catch (ClassNotFoundException e) {
+      error(new LispError("Class not found: " + className));
+      // Not reached.
+      return null;
     }
+  }
 
     private static Class javaClass(LispObject obj) {
-	return javaClass(obj, JavaClassLoader.getCurrentClassLoader());
+        return javaClass(obj, JavaClassLoader.getCurrentClassLoader());
     }
 
     // Supports Java primitive types too.
@@ -1388,7 +1456,7 @@ public final class Java
                 return Double.TYPE;
             // Not a primitive Java type.
             Class c;
-	    c = classForName(s, classLoader);
+            c = classForName(s, classLoader);
             if (c == null)
                 error(new LispError(s + " does not designate a Java class."));
 
@@ -1420,4 +1488,12 @@ public final class Java
             message = t.getClass().getName();
         return message;
     }
+
+  // FIXME: better handled as a Lisp symbol?  With a Java enum, the
+  // compiler probably has a better chance to optimize.
+  public static class Buffers {
+    public enum AllocationPolicy { PRIMITIVE_ARRAY, NIO; };
+    public static AllocationPolicy active = AllocationPolicy.NIO;
+  }
+  
 }
