@@ -1514,6 +1514,9 @@ public class Stream extends StructureObject {
         String s = sb.toString();
         if (s.indexOf('/') >= 0)
             return makeRatio(s, radix);
+        if (s.length() == 0) {
+          return error(new ReaderError("Malformed number in a #b/#o/#x/#r macro."));
+        }
         // Integer.parseInt() below handles a prefixed '-' character correctly, but
         // does not accept a prefixed '+' character, so we skip over it here
         if (s.charAt(0) == '+')
@@ -1818,6 +1821,11 @@ public class Stream extends StructureObject {
         return n;
     }
 
+    public Pathname getPathname()
+    {
+        return null;
+    }
+
     /** Puts a character back into the (underlying) stream
      *
      * @param n
@@ -2643,6 +2651,15 @@ public class Stream extends StructureObject {
         }
     };
 
+    // ### file-length
+    private static final Primitive FILE_LENGTH =
+    new Primitive("file-length", "stream") {
+        @Override
+        public LispObject execute(LispObject arg) {
+            return checkStream(arg).fileLength();
+        }
+    };
+
     // ### stream-line-number
     private static final Primitive STREAM_LINE_NUMBER =
     new Primitive("stream-line-number", PACKAGE_SYS, false, "stream") {
