@@ -2,7 +2,7 @@
  * StructureObject.java
  *
  * Copyright (C) 2003-2006 Peter Graves
- * $Id: StructureObject.java 13541 2011-08-27 23:23:24Z mevenson $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,11 @@ public class StructureObject extends LispObject
   private final StructureClass structureClass;
   final LispObject[] slots;
 
+  public StructureObject() {
+    structureClass = null;
+    slots = null;
+  }
+
   public StructureObject(Symbol symbol)
 
   {
@@ -154,16 +159,16 @@ public class StructureObject extends LispObject
   }
 
     protected int getSlotIndex(LispObject slotName) {
-	LispObject effectiveSlots = structureClass.getSlotDefinitions();
-	LispObject[] effectiveSlotsArray = effectiveSlots.copyToArray();
-	for (int i = 0; i < slots.length; i++) {
-	    SimpleVector slotDefinition = (SimpleVector) effectiveSlotsArray[i];
-	    LispObject candidateSlotName = slotDefinition.AREF(1);
-	    if(slotName == candidateSlotName) {
-		return i;
-	    }
-	}
-	return -1;
+        LispObject effectiveSlots = structureClass.getSlotDefinitions();
+        LispObject[] effectiveSlotsArray = effectiveSlots.copyToArray();
+        for (int i = 0; i < slots.length; i++) {
+            SimpleVector slotDefinition = (SimpleVector) effectiveSlotsArray[i];
+            LispObject candidateSlotName = slotDefinition.AREF(1);
+            if(slotName == candidateSlotName) {
+                return i;
+            }
+        }
+        return -1;
     }
 
   @Override
@@ -174,7 +179,7 @@ public class StructureObject extends LispObject
     if (index >= 0) {
         value = slots[index];
     } else {
-	value = UNBOUND_VALUE;
+        value = UNBOUND_VALUE;
         value = Symbol.SLOT_UNBOUND.execute(structureClass, this, slotName);
         LispThread.currentThread()._values = null;
     }
@@ -184,15 +189,15 @@ public class StructureObject extends LispObject
   public void setSlotValue(LispObject slotName, LispObject newValue) {
       final int index = getSlotIndex(slotName);
       if (index >= 0) {
-	  slots[index] = newValue;
+          slots[index] = newValue;
       } else {
-	  LispObject[] args = new LispObject[5];
-	  args[0] = structureClass;
-	  args[1] = this;
-	  args[2] = slotName;
-	  args[3] = Symbol.SETF;
-	  args[4] = newValue;
-	  Symbol.SLOT_MISSING.execute(args);
+          LispObject[] args = new LispObject[5];
+          args[0] = structureClass;
+          args[1] = this;
+          args[2] = slotName;
+          args[3] = Symbol.SETF;
+          args[4] = newValue;
+          Symbol.SLOT_MISSING.execute(args);
       }
   }
 
@@ -338,7 +343,7 @@ public class StructureObject extends LispObject
   {
     try
       {
-        return slots[index] != NIL ? true : false;
+        return slots[index] != NIL;
       }
     catch (ArrayIndexOutOfBoundsException e)
       {
@@ -517,7 +522,7 @@ public class StructureObject extends LispObject
       }
     catch (StackOverflowError e)
       {
-        error(new StorageCondition("Stack overflow."));
+        error(new StorageCondition("Stack overflow printing structure object"));
         return null; // Not reached.
       }
   }
