@@ -1,7 +1,7 @@
 ;;; compiler-tests.lisp
 ;;;
 ;;; Copyright (C) 2005 Peter Graves
-;;; $Id: compiler-tests.lisp 14245 2012-11-15 12:33:00Z mevenson $
+;;; $Id$
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -22,8 +22,9 @@
 
 (in-package #:abcl.test.lisp)
 
-(defconstant most-positive-java-long 9223372036854775807)
-(defconstant most-negative-java-long -9223372036854775808)
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defvar most-positive-java-long 9223372036854775807)
+  (defvar most-negative-java-long -9223372036854775808))
 
 #+abcl
 (assert (eql most-positive-java-long ext:most-positive-java-long))
@@ -104,7 +105,7 @@
   #.(+ most-positive-fixnum most-positive-fixnum))
 #+allegro (pushnew 'plus.3 *expected-failures*)
 
-#-clisp
+#+abcl
 (define-compiler-test plus.4
   (lambda (x y)
     (declare (type (integer #.most-negative-java-long #.most-positive-java-long) x y))
@@ -445,7 +446,7 @@
 
 ;;; ticket #147
 #+abcl
-(deftest compiler.1 
+(deftest compiler.1
     (let ((tmpfile (ext::make-temp-file))
           (original-print-case *print-case*)
           (forms `((in-package :cl-user)
@@ -465,7 +466,7 @@
 #+abcl
 (deftest compiler.2
     (let ((tmpfile (ext::make-temp-file))
-          (line "(defconstant a #.(make-array '(8 256) 
+          (line "(defconstant a #.(make-array '(8 256)
                     :element-type '(unsigned-byte 32) :initial-element 0))"))
       (with-open-file (s tmpfile :direction :output)
         (format s "~A" line))
@@ -501,3 +502,93 @@
           (compile nil '(lambda (&key args &optional x))))
       (typep error 'program-error))
   t)
+
+
+;;; ticket #606 Github
+#+abcl
+(deftest compiler.5a
+    (let ((tmpfile (ext::make-temp-file))
+          (code
+             "(defun abcl/test/lisp::cp5a ()
+               '(0)
+               (let ((x #1='(0))
+                     (y (second '#1#)))
+                 (assert (equal x '(0)))
+                 (assert (equal y '(0)))
+                 (assert (eq x y))))"))
+      (with-open-file (s tmpfile :direction :output)
+        (format s "~A" code))
+      (load (compile-file tmpfile))
+      (delete-file tmpfile)
+      (prog1 (abcl/test/lisp::cp5a)
+        (fmakunbound 'abcl/test/lisp::cp5a)))
+  nil)
+
+#+abcl
+(deftest compiler.5b
+    (let ((tmpfile (ext::make-temp-file))
+          (code
+             "(defun abcl/test/lisp::cp5b ()
+               '(0)
+               (let ((x #1='(0 #1#))
+                     (y (second '#1#)))
+                 (assert (eq x y))))"))
+      (with-open-file (s tmpfile :direction :output)
+        (format s "~A" code))
+      (load (compile-file tmpfile))
+      (delete-file tmpfile)
+      (prog1 (abcl/test/lisp::cp5b)
+        (fmakunbound 'abcl/test/lisp::cp5b)))
+  nil)
+
+#+abcl
+(deftest compiler.5c
+    (let ((tmpfile (ext::make-temp-file))
+          (code
+             "(defun abcl/test/lisp::cp5c ()
+               '(0)
+               (let ((x #1='(0))
+                     (y (second (second ''#1#))))
+                 (assert (equal x '(0)))
+                 (assert (equal y '(0)))
+                 (assert (eq x y))))"))
+      (with-open-file (s tmpfile :direction :output)
+        (format s "~A" code))
+      (load (compile-file tmpfile))
+      (delete-file tmpfile)
+      (prog1 (abcl/test/lisp::cp5c)
+        (fmakunbound 'abcl/test/lisp::cp5c)))
+  nil)
+
+#+abcl
+(deftest compiler.5d
+    (let ((tmpfile (ext::make-temp-file))
+          (code
+             "(defun abcl/test/lisp::cp5d ()
+               '(0)
+               (let ((x #1='(0))
+                     (y (second (second (second '''#1#)))))
+                 (assert (equal x '(0)))
+                 (assert (equal y '(0)))
+                 (assert (eq x y))))"))
+      (with-open-file (s tmpfile :direction :output)
+        (format s "~A" code))
+      (load (compile-file tmpfile))
+      (delete-file tmpfile)
+      (prog1 (abcl/test/lisp::cp5d)
+        (fmakunbound 'abcl/test/lisp::cp5d)))
+  nil)
+
+#+abcl
+(deftest compiler.5e
+    (let ((tmpfile (ext::make-temp-file))
+          (code
+             "(defmacro cp5e (codelist)
+               `(list '(list) ,codelist))"))
+      (with-open-file (s tmpfile :direction :output)
+        (format s "~A" code))
+      (prog1
+          (load (compile-file tmpfile))
+        (delete-file tmpfile)
+        (fmakunbound 'abcl/test/lisp::cp5e)))
+  t)
